import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Point;


public class OptionsScreen extends JFrame implements ActionListener,MouseMotionListener,MouseListener,ItemListener

{
	Screen goBack;
	PositionsPanel pP;
	Storage store;
	
	BorderLayout layout = new BorderLayout();
	FlowLayout flow = new FlowLayout();
	GridLayout grid = new GridLayout(2,1);
	
	JPanel southPanel = new JPanel();
	JPanel buttonsPanel = new JPanel();
	JPanel mousePositionPanel = new JPanel();
	MapPanel mapPanel = new MapPanel();
	
	
	JLabel noSlaves = new JLabel("Number of Slaves:",SwingConstants.LEFT);
	JLabel mousePositionLabel = new JLabel("Mouse Coordinates :  ",SwingConstants.LEFT);
	JLabel mouseAt = new JLabel(" -,-");
	
	JComboBox chooseNoSlaves;
	
	JCheckBox showRanges;
	
	JButton rangeButton;
	JButton setMasterPositionButton;
	JButton setSlavesPositionsButton;
	JButton doneButton;
	
	
	String slavesMatrix[] = {"1","2","3","4","5","6","7"};
	
	byte peerCounter =1;
	byte maxNumOfSlaves = 1;
	

	
	public OptionsScreen(Screen in , Storage sto)
	{
				
		super("Simulation Parameters");
		goBack = in;
		store = sto;
		peerCounter = -1;
		setLayout(layout);		
		chooseNoSlaves = new JComboBox(slavesMatrix);
		chooseNoSlaves.setMaximumRowCount(4);
		showRanges = new JCheckBox("Show Ranges",false);		
		rangeButton = new JButton("Set Ranges");
		setMasterPositionButton = new JButton("Place Master");
		setSlavesPositionsButton = new JButton("Place Slaves");
		doneButton = new JButton("Next >>");
		
		chooseNoSlaves.setToolTipText("Choose how many slaves will participate in the piconet");
		rangeButton.setToolTipText("Choose between 10/100m range");
		setMasterPositionButton.setToolTipText("Place the master peer in map above");
		setSlavesPositionsButton.setToolTipText("Place slave peers in map above");
		doneButton.setToolTipText("Accept current configuration");
		showRanges.setToolTipText("Choose wether you want the peers' ranges to be visible");
		
		
		mapPanel.addMouseMotionListener(this);
		mapPanel.addMouseListener(this);
		rangeButton.addActionListener(this);
		setMasterPositionButton.addActionListener(this);
		setSlavesPositionsButton.addActionListener(this);
		doneButton.addActionListener(this);
		chooseNoSlaves.addItemListener(this);
		showRanges.addItemListener(this);
		
		buttonsPanel.setLayout(flow);
		buttonsPanel.add(noSlaves);
		buttonsPanel.add(chooseNoSlaves);
		buttonsPanel.add(rangeButton);
		buttonsPanel.add(setMasterPositionButton);
		buttonsPanel.add(setSlavesPositionsButton);
		buttonsPanel.add(doneButton);
		
		mousePositionPanel.setLayout(flow);
		mousePositionPanel.add(mousePositionLabel);
		mouseAt.setForeground(Color.RED);
		mousePositionPanel.add(mouseAt);
		mousePositionPanel.add(showRanges);
		
		southPanel.setLayout(grid);
		southPanel.add(buttonsPanel);
		southPanel.add(mousePositionPanel);
		
		add(mapPanel,BorderLayout.CENTER);
		add(southPanel,BorderLayout.SOUTH);
		pP = new PositionsPanel();
		pP.setLocation(891,0);
		pP.setSize(250,350);
		pP.setResizable(false);		
		pP.setEnabled(false);
		pP.setVisible(true);
		
		
		
		
	}
	
	public void actionPerformed (ActionEvent ae)
	{
		//String command = ae.getActionCommand();
		if ( ae.getSource() == doneButton )
		{
			if ((mapPanel.peerPositions[0].getX()!=0) && (mapPanel.peerPositions[0].getY()!=0) && (mapPanel.peerPositions[1].getX()!=0) && (mapPanel.peerPositions[1].getY()!=0))
			{
				store.setPositions(mapPanel.peerPositions);
				store.setRanges(mapPanel.rangesMatrix);
				store.setPeerNum(chooseNoSlaves.getSelectedIndex()+1);	
				for (int i=0 ; i<8 ;i++){			
					System.out.print(store.peerPositions[i].getX());
					System.out.print(store.peerPositions[i].getY());			
				}
				pP.dispose();
				SimParams simParams = new SimParams(goBack,store);
				this.dispose();
				
			}
			else 
			{
				JOptionPane.showMessageDialog(OptionsScreen.this,"You must place master and at least one slave peer in order to exit configuration panel","INPUT ERROR!",JOptionPane.ERROR_MESSAGE);
			}		
						
		}
		
		else if (ae.getSource() == setMasterPositionButton )
		{
			peerCounter = 0;
			pP.resetMaster();
			mapPanel.peerPositions[0].x = 0;
			mapPanel.peerPositions[0].y = 0;
			mapPanel.repaint();
			
			
		}
		
		else if (ae.getSource() == setSlavesPositionsButton )
		{
			peerCounter = 1;
			pP.resetSlaves();
			for (int i = 1 ; i < mapPanel.peerPositions.length ; i++)
			{
				mapPanel.peerPositions[i].x = 0;
				mapPanel.peerPositions[i].y = 0;
				mapPanel.repaint();
			}
			
		}
		
		else if (ae.getSource() == rangeButton )
		{
			SetRangesFrame rangeFrame = new SetRangesFrame(maxNumOfSlaves , mapPanel);			
			rangeFrame.setLocation(891,350);
			rangeFrame.setSize(250,400);
			rangeFrame.setResizable(false);					
			rangeFrame.setVisible(true);
		
		}
		
		
	}
	
	public void mouseMoved(MouseEvent me)
	{
		mouseAt.setText(String.format("(%d , %d)",me.getX(),me.getY()));
	}
	
	public void mouseDragged(MouseEvent me) {}
	
	public void mouseClicked(MouseEvent me) 
	{
		Point p = new Point(me.getX(),me.getY());
		boolean douplicate = checkDuplicate(p);
		boolean maxSlaves = checkNumOfSlaves();
		
		if (peerCounter!=-1)
		{
			if ((!douplicate) && (!maxSlaves))
			{
				mapPanel.setPeer(p,peerCounter);
				pP.setPeerPosition(p,peerCounter);
				mapPanel.repaint();
				pP.panel.repaint();
			}
			
		}
		
		
		if (peerCounter==0) {}
		else if ((peerCounter > 0) && (!douplicate) && (!maxSlaves)) peerCounter++;
	
	}
	
	public void mousePressed(MouseEvent me) {}
	
	public void mouseReleased(MouseEvent me) {}
	
	public void mouseEntered(MouseEvent me) {}	
	
	public void mouseExited(MouseEvent me)
	{		
		mouseAt.setText(" Mouse Outside Panel");
	} 
	
	public void itemStateChanged(ItemEvent ie)
	{
		if (ie.getSource() == showRanges)
		{
			if (ie.getStateChange() == ItemEvent.SELECTED)
			{
				mapPanel.showRanges(true);
				mapPanel.repaint();
			}
			if (ie.getStateChange() == ItemEvent.DESELECTED)
			{
				mapPanel.showRanges(false);
				mapPanel.repaint();
			}		
		}
		
		else 
		{
			if (ie.getStateChange() == ItemEvent.SELECTED)
			{
				if ( maxNumOfSlaves <= (byte)(chooseNoSlaves.getSelectedIndex()+1) )
				{
					maxNumOfSlaves = (byte)(chooseNoSlaves.getSelectedIndex()+1);
				}
				else
				{
					maxNumOfSlaves = (byte)(chooseNoSlaves.getSelectedIndex()+1);
					mapPanel.removeSlaves((byte)(chooseNoSlaves.getSelectedIndex()+1));
					peerCounter = (byte)(chooseNoSlaves.getSelectedIndex()+2);
					pP.delSlaves(peerCounter);
					mapPanel.repaint();					
				}
					
				System.out.println(maxNumOfSlaves);
			}
		}
	}
	
	public boolean checkDuplicate(Point p)
	{
		for (int i = 0 ; i < mapPanel.peerPositions.length ; i++ )
		{
			if ((p.x == mapPanel.peerPositions[i].x) && (p.y == mapPanel.peerPositions[i].y))
			{
				JOptionPane.showMessageDialog(OptionsScreen.this,"Two Peers cannot be placed in the same coordinates","INPUT ERROR!",JOptionPane.ERROR_MESSAGE);
				return true;
			}			
		}
		return false;
	}
	
	public boolean checkNumOfSlaves()
	{
		System.out.println(peerCounter+" > "+maxNumOfSlaves);
		if ( peerCounter > maxNumOfSlaves)
		{
			JOptionPane.showMessageDialog(OptionsScreen.this,"You have already placed the maximum number of slaves you specified","INPUT ERROR!",JOptionPane.ERROR_MESSAGE);
			return true;
		}
		return false;
	}
	
	/*public static void main(String args[])
	{
		OptionsScreen test = new OptionsScreen(new Screen());
		test.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		test.setSize(650,590);
		test.setResizable(true);
		test.setLocation(250,0);
		test.setVisible(true);
		
	}*/
	
	
	
	
	/*
	public drawMap(Graphics g)
	{
		mapPanel.paintComponent(g);
		
		int width = mapPanel.getWidth();
		int height = mapPanel.getHeight();
		
		for (int i=0,i>width;i+=5)
		{
			g.drawLine(i,0,i,height);
		}
		
		for (int x=0,x>height;x+=5)
		{
			g.drawLine(0,x,width,x);
		}
		
		
	}*/
	/*
	public class drawMap extends JPanel
	{
		
		
		public void paintComponent( Graphics g)
		{
			super.paintComponent(g);
		
			int width = getWidth();
			int height = getHeight();
		
		}
	}*/
}