import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class Screen extends JFrame implements ActionListener
{
	
	private JPanel buttonsPanel = new JPanel(new GridLayout(5,1,5,10));
	
	private JButton optionsButton;
	private JButton startSimButton;
	private JButton statisticsButton;
	private JButton aboutButton;
	private JButton exitButton;
	
	private JLabel logoLabel;
	
	private boolean configured = false;
	private boolean executed = false;
	
	MapPanel mapPanel;
	
	SimParams simParams;
	Simulation startSim;
	Storage storage = new Storage();
	
	public Screen()
	{
		super("Bluetooth Protocol Simulator");
		//setLayout( new FlowLayout(FlowLayout.CENTER,0,10));
		
		Icon logo = new ImageIcon( getClass().getResource( "pics/logo.jpg"));
		
		logoLabel = new JLabel("Please choose one of the following:",logo,SwingConstants.CENTER);
		logoLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		logoLabel.setVerticalTextPosition(SwingConstants.BOTTOM);
				
		optionsButton = new JButton("Edit Options");
		startSimButton = new JButton("Start Simulation");
		statisticsButton = new JButton("Statistics");
		aboutButton = new JButton("About");
		exitButton = new JButton("Exit Simulator");
		
		Dimension buttonDimension = new Dimension(20,10);		
		optionsButton.setSize((int)buttonDimension.getHeight(),(int)buttonDimension.getWidth());
		statisticsButton.setPreferredSize(buttonDimension);
		aboutButton.setSize(buttonDimension);
		exitButton.setSize(buttonDimension);
		
		optionsButton.addActionListener(this);
		startSimButton.addActionListener(this);
		statisticsButton.addActionListener(this);
		aboutButton.addActionListener(this);
		exitButton.addActionListener(this);
		
		optionsButton.setToolTipText("Configure Simulator");
		startSimButton.setToolTipText("Execute Simulator");
		statisticsButton.setToolTipText("View results");
		aboutButton.setToolTipText("Credits");
		exitButton.setToolTipText("Quit Application");
		exitButton.setSize(new Dimension(90,30));
		
		add(logoLabel,BorderLayout.NORTH);
		buttonsPanel.add(optionsButton);		
		buttonsPanel.add(startSimButton);
		buttonsPanel.add(statisticsButton);
		buttonsPanel.add(aboutButton);
		buttonsPanel.add(exitButton);
		add(buttonsPanel,BorderLayout.CENTER);
				
		
		
	}
	
	
	public void keepMap( MapPanel mP)
	{
		mapPanel = mP;
	}
	
	public MapPanel getMap()
	{
		return mapPanel;
	}
	
	public void setConfigured()
	{
		configured = true;
	}
	
	public void setExecuted()
	{
		executed = true;
	}
	
	
	public void actionPerformed( ActionEvent ae)
	{
		String command = ae.getActionCommand();		
		if (command == "Exit Simulator")
		{
			System.exit(0);
		}
		else if (command == "Edit Options")
		{
			this.setEnabled(false);
			OptionsScreen editSim = new OptionsScreen(this,storage);
			editSim.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			editSim.setSize(640,640);
			editSim.setResizable(true);
			editSim.setLocation(251,0);
			editSim.setVisible(true);		
		}
		
		else if (command == "Start Simulation")
		{
			if (configured)
			{
				this.setEnabled(false);
				System.out.println("mpika");
		 		startSim = new Simulation(this,storage);		 		
				startSim.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				startSim.setSize(640,640);
				startSim.setResizable(true);
				startSim.setLocation(251,0);
				startSim.setEnabled(true);
				startSim.setVisible(true);				
			}
			else
			{
				JOptionPane.showMessageDialog(null,"Please first configure simulation properties by pressing the\"Start Simulation\" button above","Statistics Error!",JOptionPane.ERROR_MESSAGE);
			}
		}
		
		else if (command == "Statistics")
		{
			
			if (executed)
			{
				this.setEnabled(false);
				Statistics stats = new Statistics(this,storage);				
			}
			else
			{
				JOptionPane.showMessageDialog(null,"Please first execute the simulation by pressing the\"Edit Options\" button above","Configuration Error!",JOptionPane.ERROR_MESSAGE);
			}
			
		}
		
		else if (command == "About")
		{
			this.setEnabled(false);
			About about = new About(this);
		}
	}
	
}