import javax.swing.*;
import java.awt.*;
import java.lang.Math;
import java.util.concurrent.Executors;
import java.util.concurrent.ExecutorService;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JOptionPane;
import java.lang.Integer;
import java.lang.NumberFormatException;
import java.lang.NullPointerException;


public class Simulation extends JFrame implements ActionListener,ItemListener,MouseListener
{
	
	JPanel southPanel = new JPanel();
	JPanel buttonsPanel = new JPanel();
	JPanel aboveButtonsPanel = new JPanel(new GridLayout(1,2));
	JPanel runOptionsPanel = new JPanel();
	JPanel showStatusPanel = new JPanel(new FlowLayout());
		
	FlowLayout flow = new FlowLayout();
	GridLayout southGrid = new GridLayout(2,1,5,5);
	
	JLabel minutesLabel = new JLabel("minute(s)");
	JLabel runConLabel = new JLabel("Run Continuasly");
	
	JButton runButton = new JButton("Run");
	JButton pauseButton = new JButton(" Pause");
	JButton stopButton = new JButton("Stop");
	JButton closeButton = new JButton("Close");
	
	JTextField minuteField = new JTextField(2);
	
	JRadioButton runMinutes;
	JRadioButton runAlways;
	
	ButtonGroup runOption;
	
	JCheckBox showStatus = new JCheckBox("Show Status",true);
	
	SimMap simMap;
	SimClock simClock;	
	Screen screen;
	Storage store;
	
	Peer peerMatrix[];
	int delays[];
	int throughput = 0;
	int totalBytes = 0;
	int totalSecs = 0;
	
	ExecutorService application;
	ThreadHandler threadHandler = new ThreadHandler();
	
	int peerTotal = 0;
	int mode = 1;
	int timeLimit;
	
	boolean error = false;
	public Simulation(Screen in, Storage sto)
	{
		super("Bluetooth Simulation");
		
		screen = in ;
		store = sto;
		simMap = new SimMap(store);		
		simClock = new SimClock(store);
		runMinutes = new JRadioButton("Run for ",false);
		runAlways = new JRadioButton("Run Continuasly",true);
		runOption = new ButtonGroup();
		runOption.add(runMinutes);
		runOption.add(runAlways);
		runOptionsPanel.setLayout(flow);
		runOptionsPanel.add(runAlways);
		runOptionsPanel.add(runMinutes);
		runOptionsPanel.add(minuteField);
		runOptionsPanel.add(minutesLabel);
		buttonsPanel.setLayout(flow);
		buttonsPanel.add(runButton);
		//buttonsPanel.add(pauseButton);
		buttonsPanel.add(stopButton);
		buttonsPanel.add(closeButton);
		showStatusPanel.add(showStatus);		
		aboveButtonsPanel.add(runOptionsPanel);
		aboveButtonsPanel.add(showStatusPanel);
		
		
		runAlways.addItemListener(this);
		runMinutes.addItemListener(this);
		minuteField.setEnabled(false);
		showStatus.addItemListener(this);
		runButton.addActionListener(this);
		pauseButton.addActionListener(this);
		stopButton.addActionListener(this);
		closeButton.addActionListener(this);
		
		stopButton.setEnabled(false);
		closeButton.setEnabled(false);
		
		southPanel.setLayout(southGrid);
		southPanel.add(aboveButtonsPanel);
		southPanel.add(buttonsPanel);
		this.add(simClock,BorderLayout.NORTH);
		this.add(simMap,BorderLayout.CENTER);
		this.add(southPanel,BorderLayout.SOUTH);
		
		simMap.addMouseListener(this);
	}
		
		
	public void setPeerStatus(int pN , int st)
	{
		simMap.status[pN] = st;
		simMap.repaint();
	}
	
	public boolean checkPeersInRange(Peer p)
	{
		Point[] positions = new Point[8];
		int[] ranges = new int[8];
		boolean flag = false;
		positions = store.getPositions();
		ranges = store.getRanges();
		for (int i=1 ; i < positions.length ; i++ )
		{
			if (Math.abs(Math.sqrt(Math.pow(positions[i].getX()-positions[0].getX(),2)+Math.pow(positions[i].getY()-positions[0].getY(),2)))<=simMap.rangesMatrix[p.getPeerNum()] && 
				Math.abs(Math.sqrt(Math.pow(positions[i].getX()-positions[0].getX(),2)+Math.pow(positions[i].getY()-positions[0].getY(),2)))<=simMap.rangesMatrix[i])
			{
				p.setPeerInRange(i);				
				flag = true;
			}
		}
		
		return flag;		
	}
	
	public boolean checkMasterInRange(Peer p)
	{
		Point[] positions = new Point[8];
		int[] ranges = new int[8];
		boolean flag = false;
		positions = store.getPositions();
		ranges = store.getRanges();
		if (Math.abs(Math.sqrt(Math.pow(positions[p.getPeerNum()].getX()-positions[0].getX(),2)+Math.pow(positions[p.getPeerNum()].getY()-positions[0].getY(),2)))<=simMap.rangesMatrix[p.getPeerNum()])
		{
			p.setMasterInRange();
			flag = true;
		}
		
		return flag;
	
	}
	
	public float calculateDistance(int i)
	{
		
		Point[] positions = new Point[8];
		positions = store.getPositions();
		float distance =(float) (Math.abs(Math.sqrt(Math.pow(positions[i].getX()-positions[0].getX(),2)+Math.pow(positions[i].getY()-positions[0].getY(),2))));
		
		return distance;
	}
	
	public void setInquired()
	{
		peerMatrix[0].incPeersInquired();
		//System.out.println("I was fucking inquired");
	}
	
	public void setPaged()
	{
		peerMatrix[0].incPeersPaged();
	}
	
	public void getPeerNum()
	{
		Point[] positions = new Point[8];
		int[] ranges = new int[8];
		boolean flag = false;
		positions = store.getPositions();
		ranges = store.getRanges();
		for (int i=0 ; i < positions.length ; i++ )
		{
			if ((positions[i].getX()!=0) || (positions[i].getY()!=0))  peerTotal++;
		}
	}
	
	public void setColor(Color color,int peerNum)
	{
		simMap.setColor(color,peerNum);
	}
	
	public int getTimeLimit()
	{
		return simClock.getCurrentMinute();
	}
	
	public boolean checkIfPeerWantsToTransmit(int p)
	{
		if (peerMatrix[p].bufferEmpty()) return false;
			else return true;
	}
	
	public void orderTransmission(int p)
	{
		peerMatrix[p].okToTransmit();
	}
	
	public void setConnected(int p)
	{
		peerMatrix[p].setConnected();
	}
	
	public void setN(int i)
	{
		store.setPeerNum(i);
	}
	
	public void drawPacket(int p)
	{
		simMap.transmission(p);
	}
	
	public void timesUp()
	{
		closeButton.setEnabled(true);
		stopButton.setEnabled(false);
		simClock.setSpecific(timeLimit);
		for (int i=0 ; i < peerTotal ; i++ ) 
		{
			this.setPeerStatus(i,0);
			delays[i] = peerMatrix[i].getDelay();
			System.out.println("peer no"+i+"\tdelay :"+delays[i]);
			totalBytes+=peerMatrix[i].getBytes();
			peerMatrix[i].stop();			
		}
		simClock.toggleRunning();
		totalSecs = simClock.getTotalSecs();
		store.setDelay(calculateDelay());
		store.setThroughput(calculateThroughput());	
		application.shutdown();
		JOptionPane.showMessageDialog(null,"Time's up! Simulation Terminated! Please visit the Statistics section for a brief report regarding simulation results.","Simulation terminated!",JOptionPane.INFORMATION_MESSAGE);			

	}
	
	private int calculateDelay()
	{
		int tempDelay = 0;
		for (int i = 0 ; i < delays.length ; i++)
		{
			tempDelay+=delays[i];
		}
		System.out.println("tempDelay :"+tempDelay+"\tpeerTotal :"+peerTotal);
		return tempDelay/peerTotal;
	}
	
	private int calculateThroughput()
	{
		return (totalBytes*8)/totalSecs;
	}
	
	public void actionPerformed (ActionEvent ae)
	{
		if ( ae.getSource() == runButton )
		{
			
			runButton.setEnabled(false);
			stopButton.setEnabled(true);
			getPeerNum();
			simClock.startClock();
		    application = Executors.newFixedThreadPool(peerTotal+1);
			peerMatrix = new Peer[peerTotal];
			delays = new int[peerTotal];
			//System.out.println("peerTotal is "+peerTotal);
			if (mode == 2)
				{
					try
					{					
						timeLimit = Integer.parseInt(minuteField.getText());
					}
					
					catch (NullPointerException npe)
					{
						JOptionPane.showMessageDialog(null,"Please enter an integer value in the \"Run for\" field","Input Error!",JOptionPane.ERROR_MESSAGE);
						error = true;
						runButton.setEnabled(true);
						stopButton.setEnabled(false);
					}
					
					catch (NumberFormatException nfe)
					{
						JOptionPane.showMessageDialog(null,"Please enter only integers in the \"Run for\" field","Input Error!",JOptionPane.ERROR_MESSAGE);
						error = true;
						runButton.setEnabled(true);
						stopButton.setEnabled(false);						
					}
					if (timeLimit == 0 ) JOptionPane.showMessageDialog(null,"Please enter an integer greater than 0 in the \"Run for\" field","Input Error!",JOptionPane.ERROR_MESSAGE);
					else if ((timeLimit != 0) && (error == false))
					{
						JOptionPane.showMessageDialog(null,"While simulation in progress click on any of the peers to view critical information about it!","Peer Statistics",JOptionPane.INFORMATION_MESSAGE);	
					for (int i=0 ; i < peerTotal ; i++ ) 
					{
						peerMatrix[i] = new Peer(i,this,threadHandler,timeLimit,simClock,store);
						//System.out.println("I created peer no"+i);
						try
						{
								application.execute(peerMatrix[i]);
						}
						catch (Exception exception)
						{
							exception.printStackTrace();
						}
					}
					try
					{
						application.execute(simClock);				
					}
					catch (Exception exception)
					{
						exception.printStackTrace();
					}
						
			
					}//end else
			}//end mode=2
			else if (mode==1)
			{
				JOptionPane.showMessageDialog(null,"While simulation in progress click on any of the peers to view critical information about it!","Peer Statistics",JOptionPane.INFORMATION_MESSAGE);	
				for (int i=0 ; i < peerTotal ; i++ ) 
				{
					peerMatrix[i] = new Peer(i,this,threadHandler,store);
					//System.out.println("I created peer no"+i);
					try
					{
						application.execute(peerMatrix[i]);
					}
					catch (Exception exception)
					{
						exception.printStackTrace();
					}
				}
				try
				{
					application.execute(simClock);
					
				}
				catch (Exception exception)
				{	
					exception.printStackTrace();
				}
				
			}
			
					
			//application.shutdown();
		} //end if runButton
	
		else if ( ae.getSource() == stopButton )
			{
				closeButton.setEnabled(true);
				stopButton.setEnabled(false);
				for (int i=0 ; i < peerTotal ; i++ ) 
				{
					this.setPeerStatus(i,0);
					delays[i] = peerMatrix[i].getDelay();
					System.out.println("peer no"+i+"\tdelay :"+delays[i]);
					totalBytes+=peerMatrix[i].getBytes();
					peerMatrix[i].stop();			
				}				
				simClock.toggleRunning();
				totalSecs = simClock.getTotalSecs();
				store.setDelay(calculateDelay());
				store.setThroughput(calculateThroughput());				
				application.shutdown();
				JOptionPane.showMessageDialog(null,"Simulation was terminated by the user. Please visit the Statistics section for a brief report on the simulation results.","Simulation stopped!",JOptionPane.INFORMATION_MESSAGE);
			}
		
		else if ( ae.getSource() == closeButton )
			{
			
				screen.setExecuted();
				screen.setEnabled(true);
				screen.toFront();
				this.dispose();
				
			}
		
		
		else if ( ae.getSource() == pauseButton )
			{			
							
				String command = ae.getActionCommand();
				if (command==" Pause")
				{
					//simClock.pause();
					for (int i=0 ; i < peerTotal ; i++ ) 
					{
						peerMatrix[i].pause();
					}
					pauseButton.setText("Resume");
				}
				else if (command=="Resume")
				{
					//simClock.resume();
					//for (int i=0 ; i < peerTotal ; i++ ) 
					//{
					//	peerMatrix[i].resume();
					//}
					threadHandler.resume();
					pauseButton.setText(" Pause");
				}
				//System.out.println(Thread.currentThread().getName()+" "+Thread.currentThread().getId());					
				/*for (int i=0 ; i < peerTotal ; i++ ) 
				{
					peerMatrix[i].wait();
				}*/			
				
		
			}
	
	}
	
	public void itemStateChanged(ItemEvent ie)
	{
		if (ie.getSource() == runMinutes)
		{
			if (ie.getStateChange() == ItemEvent.SELECTED)
			{
				minuteField.setEnabled(true);
				mode = 2; 				
			}
			else if (ie.getStateChange() == ItemEvent.DESELECTED)
			{
					minuteField.setEnabled(false);
					mode = 1; 
			}
		}
		else if (ie.getSource() == showStatus)
		{
			if (ie.getStateChange() == ItemEvent.SELECTED)
			{
				simMap.setShowStatus(true);
			}
			else if (ie.getStateChange() == ItemEvent.DESELECTED)
			{
				simMap.setShowStatus(false);
			}
			
		}
	}
	
	public void mouseClicked(MouseEvent me) 
	{
		
		Point p = new Point(me.getX(),me.getY());
		for (int i = 0 ; i < 8 ; i++)
		{
			if ((Math.abs(simMap.getPeerPosition(i).getX()-p.getX())<10) && (Math.abs(simMap.getPeerPosition(i).getY()-p.getY())<10))
			{
				peerMatrix[i].showStats();
			}
			
		}
		
	}
	
	public void mousePressed(MouseEvent me) {}
	
	public void mouseReleased(MouseEvent me) {}
	
	public void mouseEntered(MouseEvent me) {}	
	
	public void mouseExited(MouseEvent me) {} 
	
}